﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DuckEmulator;
using DuckEmulator.HabboHotel.Inside;
using DuckEmulator.HabboHotel.Inside.Core;
using DuckEmulator.HabboHotel.Inside.NET;
using System.Security.Cryptography; 

using DuckEmulator.HabboHotel.Outside.Achievements;
using DuckEmulator.HabboHotel.Outside.Clients;
using DuckEmulator.HabboHotel.Outside.Catalogue;
using DuckEmulator.HabboHotel.Outside.Navigator;
using DuckEmulator.HabboHotel.Outside.Rooms;
using DuckEmulator.HabboHotel.Outside.Items;
using DuckEmulator.HabboHotel.Outside.Misc;
using DuckEmulator.Security;
using DuckEmulator.HabboHotel.Outside.Rooms.Trade;
using DuckEmulator.HabboHotel.Inside.Storage.MySql.Database.Interfaces;
using DuckEmulator.HabboHotel.Inside.Storage.MySql.Database;
using System.Diagnostics;
using DuckEmulator.HabboHotel.Inside.Licence;
using System.Threading;
using DuckEmulator.HabboHotel.Outside.Quests;

namespace DuckEmulator
{
    sealed class DuckEnvironment
    {
        internal static Int32 PacketNumber = 3;
        internal static Boolean SHOW_PACKETS = true;

        #region Struct
        internal static Boolean Started = false;
        internal static ConfigManager ConfigManager = new ConfigManager();
        internal static SocketListener SocketListener;
        internal static RemoteSocket RemoteSocket;
        internal static DatabaseManager DatabaseManager;

        internal static AchievementManager AchievementManager;
        internal static ClientManager ClientManager;
        internal static CatalogueManager CatalogueManager;
        internal static NavigatorManager NavigatorManager;
        internal static RoomManager RoomManager;
        internal static ItemManager ItemManager;
        internal static PixelManager PixelManager;
        internal static Global Global;
        internal static TradeManager TradeManager;
        internal static QuestManager QuestManager;

        internal static DateTime StartedBuild = DateTime.Now;

        internal static LicenceManager LicenceManager;

        internal static LicenceState LicenceState = LicenceState.NotFound;
        internal static SecurityRandom SecurityRandom = new SecurityRandom();
        #endregion

        internal static void Start()
        {
            LicenceManager = new LicenceManager();

            Thread.Sleep(500); // Not to fast Licence_Waiting.

            Console.Clear();

            Console.WriteLine("");
            Console.WriteLine("     ______            _    _____                _             _");
            Console.WriteLine("     |  _  \\          | |  |  ___|              | |           | |");
            Console.WriteLine("     | | | |_   _  ___| | _| |__ _ __ ___   __ _| |_   _  __ _| |_ ___  _ __");
            Console.WriteLine("     | | | | | | |/ __| |/ /  __| '_ ` _ \\ / _` | | | | |/ _` | __/ _ \\| '__|");
            Console.WriteLine("     | |/ /| |_| | (__|   <| |__| | | | | | (_| | | |_| | (_| | || (_) | |");
            Console.WriteLine("     |___/  \\__,_|\\___|_|\\_\\____/_| |_| |_|\\__,_|_|\\__,_|\\__,_|\\__\\___/|_|");
            Console.WriteLine("");
            Console.WriteLine("     ---------------------------------------------------------------------");
            Console.ForegroundColor = ConsoleColor.DarkCyan;
            Console.WriteLine("             Duck Emulator R63 Pack " + PacketNumber  + "  | Licence State: " + ((LicenceState == HabboHotel.Inside.Licence.LicenceState.NotFound || LicenceState == HabboHotel.Inside.Licence.LicenceState.InUse) ? "Invalid" : "Accepted"));
            Console.WriteLine(" ");

            if (LicenceState == HabboHotel.Inside.Licence.LicenceState.NotFound)
            {
                WriteLine("Your Licence it not Accepted, maby its Outdated or not right typed.", ConsoleColor.Red);
                Console.Title = "DuckEmulator - Invalid Licence.";
                return;
            }
            else if (LicenceState == HabboHotel.Inside.Licence.LicenceState.InUse)
            {
                WriteLine("Your Licence it not Accepted, its already in use.", ConsoleColor.Red);
                Console.Title = "DuckEmulator - Licence in use.";
                return;
            }
            else if (LicenceState == HabboHotel.Inside.Licence.LicenceState.ServerDead)
            {
                WriteLine("Your Licence could not be verified, because the server is dead.", ConsoleColor.Red);
                Console.Title = "DuckEmulator - Server dead.";
                return;
            }

            SocketListener = new SocketListener();
            RemoteSocket = new RemoteSocket();

            DatabaseManager = new DatabaseManager(uint.Parse(ConfigManager.KeyVals["sql.maxpool"]), int.Parse(ConfigManager.KeyVals["sql.clients"]));

            DatabaseManager.setServerDetails(
                ConfigManager.KeyVals["sql.host"],
                uint.Parse(ConfigManager.KeyVals["sql.port"]),
                ConfigManager.KeyVals["sql.username"],
                ConfigManager.KeyVals["sql.password"],
                ConfigManager.KeyVals["sql.database"]);

            if (!DatabaseManager.init())
            {
                WriteLine("DatabaseManager could not connect to database: " + ConfigManager.KeyVals["sql.database"], ConsoleColor.Red);
                return;
            }

            else
            {
                WriteLine("DatabaseManager connected to database: " + ConfigManager.KeyVals["sql.database"], ConsoleColor.Yellow);
            }

            ItemManager = new ItemManager();
            AchievementManager = new AchievementManager(); 
            ClientManager = new ClientManager(); 
            CatalogueManager = new CatalogueManager(); 
            RoomManager = new RoomManager();
            NavigatorManager = new NavigatorManager();
            PixelManager = new PixelManager();
            Global = new Global();
            TradeManager = new TradeManager();
            QuestManager = new QuestManager();

            if (!Started) Started = true;

            Console.Beep();

            WriteLine("Started environment in " + (int)(DateTime.Now - StartedBuild).TotalSeconds + " sec.", ConsoleColor.White);
        }

        internal static String CleanInjections(string Input)
        {
            Input = Input.Replace(Convert.ToChar(1), ' ');
            Input = Input.Replace(Convert.ToChar(2), ' ');
            Input = Input.Replace(Convert.ToChar(3), ' ');
            Input = Input.Replace(Convert.ToChar(9), ' ');

            return (String)Input;
        }

        internal static void WriteLine(string Line)
        {
            WriteLine(Line, ConsoleColor.Gray);
        }

        internal static void WriteLine(string Line, ConsoleColor Color)
        {
            Console.ForegroundColor = Color;
            Console.WriteLine(" [" + DateTime.Now.ToShortTimeString() + "] -> " + Line);
            Console.ForegroundColor = ConsoleColor.Gray;
        }

        internal static void WriteLine(string Line, bool Into)
        {
            Console.WriteLine("[" + DateTime.Now.ToString() + "] " + ((Into) ? "<-" : "->") + " " + Line);
        }


        internal static string DecodeHex(Object Hex)
        {
            byte[] letters = new byte[Hex.ToString().Length / 2];

            ASCIIEncoding encoder = new ASCIIEncoding();

            for (int i = 0; i < (Hex.ToString().Length - 1); i += 2)
            {
                letters[i / 2] = Convert.ToByte(Hex.ToString().Substring(i, 2), 16);
            }

            return encoder.GetString(letters);
        }

        internal static void Dispose()
        {
            if (!Started) return;

            if (LicenceManager != null)
            {
                LicenceManager.Dispose();
                LicenceManager = null;
            }

            if (PixelManager != null)
            {
                PixelManager.Dispose();
                PixelManager = null;
            }

            if (ClientManager != null)
            {
                ClientManager.Dispose();
                ClientManager = null;
            }

            if (SocketListener != null)
            {
                SocketListener.Dispose();
                SocketListener = null;
            }

            if (RoomManager != null)
            {
                RoomManager.Dispose();
                RoomManager = null;
            }

            if (CatalogueManager != null)
            {
                CatalogueManager = null;
            }

            if (DatabaseManager != null)
            {
                DatabaseManager.destroy();
                DatabaseManager = null;
            }

            if (AchievementManager != null)
            {
                AchievementManager = null;
            }

            if (TradeManager != null)
            {
                TradeManager = null;
            }

            if (QuestManager != null)
            {
                QuestManager = null;
            }

            if (Global != null)
            {
                Global = null;
            }

            if (Started) Started = false;
        }

        internal static string GetXMLProxy()
        {
            return "<?xml version=\"1.0\"?>\r\n" +
                   "<!DOCTYPE cross-domain-policy SYSTEM \"/xml/dtds/cross-domain-policy.dtd\">\r\n" +
                   "<cross-domain-policy>\r\n" +
                   "<allow-access-from domain=\"*\" to-ports=\"1-31111\" />\r\n" +
                   "</cross-domain-policy>\x0";
        }

        internal static string GetDetailsString()
        {
            StringBuilder Builder = new StringBuilder();

            Builder.AppendLine("DuckEmulator related stats");
            Builder.AppendLine();
            Builder.AppendLine(GetRunningTime(false));
            Builder.AppendLine(GetRamUsage());
            Builder.AppendLine("Users alive: " + ClientManager.Clients.Count);
            Builder.AppendLine("Rooms alive: " + RoomManager.AliveRooms.Count);
            Builder.AppendLine("Catalogue pages: " + CatalogueManager.Pages.Count);
            Builder.AppendLine("Room models: " + RoomManager.RoomModels.Count);
            Builder.AppendLine("Achievements: " + AchievementManager.Achievements.Count);
            Builder.AppendLine("Pixels refill in: " + (5 - (DateTime.Now - PixelManager.LastUpdated).TotalMinutes) + " mins.");
            Builder.AppendLine("Items: " + ItemManager.Items.Count);
            Builder.AppendLine("Public rooms: " +NavigatorManager.PublicRooms.Count);
            Builder.AppendLine("Public categorys: " + NavigatorManager.PublicCats.Count);
            return Builder.ToString();
        }

        internal static string GetRunningTime(bool Header)
        {
            TimeSpan D = (DateTime.Now - StartedBuild);

            StringBuilder Builder = new StringBuilder();

            Builder.Append("Running for: ");
            Builder.Append(D.Days + " d, ");
            Builder.Append(D.Hours + " h, ");
            Builder.Append(D.Minutes + " m and ");
            Builder.Append(D.Seconds + " s.");

            if (Header)
            {
                Builder.Append(" | ");
                Builder.Append(GetRamUsage());
            }

            return Builder.ToString();
        }

        internal static string GetRamUsage()
        {
            return "RAM : " + Environment.WorkingSet / 2300;
        }

        internal static IQueryAdapter GetAdapter()
        {
            return DatabaseManager.getQueryreactor();
        }
    }
}
