﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using DuckEmulator.GUI;
using DuckEmulator;
using DuckEmulator.GUI.Actions.Broadcast;
using DuckEmulator.GUI.Stats;
using DuckEmulator.GUI.Actions.Handle;
using System.Threading;

namespace DuckEmulator
{
    partial class Interface : Form
    {
        internal SplashScreen Splash;
        internal DateTime StartedTime = DateTime.Now;
        private string LastRelease = "R63";

        internal Interface()
        {
            InitializeComponent();

            DuckEnvironment.GUI = this;

            UpdateDate();
            UpdateGame();
            UpdateDetails();
        }

        private void UpdateDetails()
        {
            this.Version.Text = "R" + System.Reflection.Assembly.GetExecutingAssembly().GetName().Version.ToString().Split('.')[0];
            this.SwfVersion.Text = LastRelease;

            if (SwfVersion.Text != Version.Text)
            {
                Version.ForeColor = Color.Red;
            }
        }

        private void UpdateDate()
        {
            this.Date.Text = DateTime.Now.ToShortDateString();

            if (DateTime.Now.Month == 10 && DateTime.Now.Day == 31)
            {
                HollydayImage.Image = DuckEmulator.Properties.Resources.HalloweenZombie;
                HollydayImage.Visible = true;
            }

            this.UpdateTime();
        }

        internal void UpdateGame()
        {
            if (DuckEnvironment.ConfigManager.KeyVals["games.enabled"] == "true")
                this.GamesEnabled.Visible = true;
            else this.GamesEnabled.Visible = false;
        }

        private void UpdateTime()
        {
            this.DayName.Text = DateTime.Now.DayOfWeek.ToString() + " - " + DateTime.Now.ToShortTimeString();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (!DuckEnvironment.Started)
            {
                Loading.Visible = true;
                Interactor.Text = "Stop Emulator";

                RefreshButton.Enabled = true;
                Warningmessage.Visible = true;

                Worker.RunWorkerAsync();
            }

            else
            {
                Loading.Visible = false;
                Interactor.Text = "Start Emulator";

                RefreshButton.Enabled = false;
                Warningmessage.Visible = false;

                DuckEnvironment.Dispose();
            }
        }

        private void CloseEnvironment_Click(object sender, EventArgs e)
        {
            if (DuckEnvironment.Started)
            {
                DuckEnvironment.Dispose();
            }

            this.Visible = false;
            Splash.Exit();
        }

        private void Minimize_Click(object sender, EventArgs e)
        {
            this.Visible = false;
            this.TrayIcon.Visible = true;
            this.TrayIcon.ShowBalloonTip(10, "DuckEmulator", "Double-click here to restore your duckEmu.", ToolTipIcon.Info);
        }

        private void Icon_Click(object sender, EventArgs e)
        {
            this.Visible = true;
            this.TrayIcon.Visible = false;
        }

        private void settingsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new Settings().ShowDialog();
        }

        private void helpBrowserToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new HelpBrowser().ShowDialog();
        }

        private void Help_Click(object sender, EventArgs e)
        {
            new HelpBrowser().ShowDialog();
        }

        private void ErrorWarning_Click(object sender, EventArgs e)
        {
            new ErrorEvents().ShowDialog();
        }

        private void errorListenerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new ErrorEvents().ShowDialog();
        }

        private delegate void Write(string Line);

        internal void WriteLine(string Line)
        {
            this.Invoke(new Write(LinerWrite), new object[] { Line });
        }

        void LinerWrite(string Line)
        {
            if (Line == null) return;

            Logging.AppendText(" " + Line);
            Logging.AppendText(Environment.NewLine);
            Logging.ScrollToCaret();
            Logging.Refresh();
        }

        private void Help_MouseHover(object sender, EventArgs e)
        {
            this.help_hower.Visible = true;
        }

        private void Help_MouseLeave(object sender, EventArgs e)
        {
            this.help_hower.Visible = false;
        }

        private void Hotel_MouseHover(object sender, EventArgs e)
        {
            this.hotel_hower.Visible = true;
        }

        private void Hotel_MouseLeave(object sender, EventArgs e)
        {
            this.hotel_hower.Visible = false;
        }

        private void Date_MouseHover(object sender, EventArgs e)
        {
            this.UpdateTime();
            this.DayName.Visible = true;
        }

        private void Date_MouseLeave(object sender, EventArgs e)
        {
            this.DayName.Visible = false;
        }

        private void hotelAlertToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new Hotelmessage().ShowDialog();
        }

        private void creditsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new Credits().ShowDialog();
        }

        private void runtimeToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void usersToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new Users().ShowDialog();
        }

        private void globalStatsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new Global().ShowDialog();
        }

        private void GamesEnabled_MouseHover(object sender, EventArgs e)
        {
            this.GamesText.Visible = true;
        }

        private void GamesEnabled_MouseLeave(object sender, EventArgs e)
        {
            this.GamesText.Visible = false;
        }

        private void RefreshButton_Click(object sender, EventArgs e)
        {
            if (!DuckEnvironment.Started) return;

            DuckEnvironment.AchievementManager.LoadAchievements();
            DuckEnvironment.NavigatorManager.LoadPubCats();
            DuckEnvironment.NavigatorManager.LoadPubRooms();
            DuckEnvironment.ItemManager.LoadFurni();
            DuckEnvironment.CatalogueManager.LoadPages();
            DuckEnvironment.RoomManager.LoadRoomModels();
        }

        private void RefreshButton_MouseHover(object sender, EventArgs e)
        {
            if (!DuckEnvironment.Started) return;

            ResfreshHower.Visible = true;
        }

        private void RefreshButton_MouseLeave(object sender, EventArgs e)
        {
            ResfreshHower.Visible = false;
        }

        private void RunTime_MouseHover(object sender, EventArgs e)
        {
            RunTimeText.Text = (DateTime.Now - StartedTime).ToString().Split('.')[0];
            RunTimeText.Visible = true;
        }

        private void RunTime_MouseLeave(object sender, EventArgs e)
        {
            RunTimeText.Visible = false;
        }

        private void ServDetails_MouseHover(object sender, EventArgs e)
        {
            ServerDetails.Visible = true;
        }

        private void ServDetails_MouseLeave(object sender, EventArgs e)
        {
            ServerDetails.Visible = false;
        }

        private void pixelsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new Pixels().ShowDialog();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show("DuckEmulator is the fastest, advanced emulator ever! Its an multi-threaded and cached emualtor that's created by Breakz0ne himself. Due running-time you can manage users with a advanced GUI. The GUI only gets 2MB of memory usage. DuckEmulator his database-handling is thinked about, and lagg-free.");
        }

        private void Settings_Click(object sender, EventArgs e)
        {
            new Settings().ShowDialog();
        }

        private void badgesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new Badges().ShowDialog();
        }

        internal void Report(int e)
        {
            Worker.ReportProgress(e);
        }

        private void roomsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new Rooms().ShowDialog();
        }

        private void Worker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            int i = e.ProgressPercentage;

            if (i < 0 || i > 100) return;

            Progress.Value = i;
            Progress.Update();
        }

        internal delegate void Do();

        internal void UpdateBar()
        {
            this.Invoke(new Do(bar));
        }

        void bar()
        {
            Progress.Style = ProgressBarStyle.Marquee;
        }

        private void Worker_DoWork(object sender, DoWorkEventArgs e)
        {
            DuckEnvironment.Start();
        }

        private void Progress_Validated(object sender, EventArgs e)
        {
            Progress.Style = ProgressBarStyle.Marquee;
        }
    }
}
