﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Text.RegularExpressions;

namespace DuckEmulator.GUI
{
    partial class Settings : Form
    {
        internal Settings()
        {
            InitializeComponent();

            ParseIntoForm();

            if (DuckEnvironment.Started)
            {
                sock_ip.Enabled = false;
                sock_port.Enabled = false;
                sock_backlog.Enabled = false;

                sql_clients.Enabled = false;
                sql_database.Enabled = false;
                sql_maxpool.Enabled = false;
                sql_host.Enabled = false;
                sql_port.Enabled = false;
                sql_clients.Enabled = false;
                sql_password.Enabled = false;
                sql_username.Enabled = false;
                games_enabled.Enabled = false;
                set_default.Enabled = true;
            }
        }

        private string Path
        {
            get { return DuckEnvironment.ConfigManager.Path; }
        }

        private Boolean CheckFile
        {
            get { return DuckEnvironment.ConfigManager.CheckFile; }
        }

        private Dictionary<string, string> KeyVals
        {
            get
            {
                return DuckEnvironment.ConfigManager.KeyVals;
            }
        }

        private Dictionary<string, string> GetNewest()
        {
            var Dic = new Dictionary<string, string>();

            Dic.Add("socket.ip", sock_ip.Text);
            Dic.Add("socket.port", sock_port.Text);
            Dic.Add("socket.backlog", sock_backlog.Text);

            Dic.Add("interface.site", interface_site.Text);

            Dic.Add("sql.host", sql_host.Text);
            Dic.Add("sql.port", sql_port.Text);
            Dic.Add("sql.username", sql_username.Text);
            Dic.Add("sql.password", sql_password.Text);
            Dic.Add("sql.database", sql_database.Text);
            Dic.Add("sql.clients", sql_clients.Text);
            Dic.Add("sql.maxpool", sql_maxpool.Text);

            Dic.Add("games.enabled", (games_enabled.Checked) ? "true" : "false");
            return Dic;
        }

        private void ParseIntoForm()
        {
            if (KeyVals.Count > 0)
            {
                foreach (KeyValuePair<string, string> kvp in KeyVals)
                {
                    switch (kvp.Key)
                    {
                        case "socket.ip":
                            {
                                this.sock_ip.Text = kvp.Value;
                            }
                            break;
                        case "socket.port":
                            {
                                this.sock_port.Text = kvp.Value;
                            }
                            break;
                        case "socket.backlog":
                            {
                                this.sock_backlog.Text = kvp.Value;
                            }
                            break;
                        case "interface.site":
                            {
                                this.interface_site.Text = kvp.Value;
                            }
                            break;

                        case "sql.host":
                            {
                                this.sql_host.Text = kvp.Value;
                            }
                            break;
                        case "sql.port":
                            {
                                this.sql_port.Text = kvp.Value;
                            }
                            break;

                        case "sql.username":
                            {
                                this.sql_username.Text = kvp.Value;
                            }
                            break;

                        case "sql.password":
                            {
                                this.sql_password.Text = kvp.Value;
                            }
                            break;
                        case "sql.database":
                            {
                                this.sql_database.Text = kvp.Value;
                            }
                            break;
                        case "sql.clients":
                            {
                                this.sql_clients.Text = kvp.Value;
                            }
                            break;
                        case "sql.maxpool":
                            {
                                this.sql_maxpool.Text = kvp.Value;
                            }
                            break;  
                        case "games.enabled":
                            {
                                this.games_enabled.Checked = (kvp.Value == "true") ? true : false;
                            }
                            break;
                    }
                }
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (!CheckFile)
            {
                return;
            }

            StreamReader reader = new StreamReader(Path);
            string content = reader.ReadToEnd();
            reader.Close();

            foreach (KeyValuePair<string, string> kvp in GetNewest())
            {
                string Line = kvp.Key + "=" + kvp.Value;
                string OldLine = kvp.Key + "=" + KeyVals[kvp.Key];

                content = content.Replace(OldLine, Line);
            }

            DuckEnvironment.ConfigManager.KeyVals = GetNewest();

            StreamWriter writer = new StreamWriter(Path);
            writer.Write(content);
            writer.Close();

            MessageBox.Show("Updated succesfully!");
            DuckEnvironment.GUI.UpdateGame();
            this.Dispose();
        }

        private void sql_maxpool_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar > 31 && (e.KeyChar < '0' || e.KeyChar > '9'))
            {
                e.Handled = true;
            }
        }

        private void sql_clients_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar > 31 && (e.KeyChar < '0' || e.KeyChar > '9'))
            {
                e.Handled = true;
            }
        }

        private void set_default_Click_1(object sender, EventArgs e)
        {
            this.sql_host.Text = "locahost";
            this.sql_database.Text = "duckdb";
            this.sql_maxpool.Text = "30";
            this.sql_clients.Text = "5";
            this.sql_password.Text = "";
            this.sql_username.Text = "root";
        }
    }
}
