﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Net.Sockets;
using DuckEmulator.HabboHotel.Outside.Clients;
using DuckEmulator.Security;
using System.Diagnostics;
using System.IO;

namespace DuckEmulator.HabboHotel.Inside.NET
{
    class SocketListener
    {
        private Socket fSocket;
        private AsyncCallback fAsync;
        private Object fState = new Object();

        private SocketFactory fFactory = new SocketFactory();

        private SecurityList<string> fBlockedIps;

        private SecurityList<string> fDossLocker;

        internal Boolean IsAlive
        {
            get
            {
                return (GetServer() != null);
            }
        }

        internal SocketListener()
        {
            fAsync = new AsyncCallback(Invoke);

            fSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);

            try
            {
                GetServer().Bind(new IPEndPoint(
                    IPAddress.Parse(DuckEnvironment.ConfigManager.KeyVals["socket.ip"]),
                    int.Parse(DuckEnvironment.ConfigManager.KeyVals["socket.port"])));
            }
            catch (Exception e)
            {
                DuckEnvironment.WriteLine("ERROR: " + e.Message, ConsoleColor.Red);
                return;
            }

            DuckEnvironment.WriteLine("SocketListener listening on: " + GetServer().LocalEndPoint, ConsoleColor.Yellow);

            GetServer().Listen(int.Parse(DuckEnvironment.ConfigManager.KeyVals["socket.backlog"]));

            fState = GetServer().BeginAccept(GetAsync(), fState).AsyncState;

            fDossLocker = new SecurityList<string>();
            fBlockedIps = new SecurityList<string>();

            if (File.Exists("blocked_ips.txt"))
            {
                foreach (string Line in File.ReadAllLines("blocked_ips.txt"))
                {
                    if (Line != null && Line.Length > 0 && Line.Contains('.'))
                    {
                        fBlockedIps.Add(Line);
                    }
                }

                DuckEnvironment.WriteLine("Loaded " + fBlockedIps.Count + " blocked IPs.", ConsoleColor.Yellow);
            }
            else
            {
                File.Create("blocked_ips.txt");
            }
        }

        private void Invoke(IAsyncResult Result)
        {
            if (!IsAlive || !Result.AsyncState.Equals(fState))
            {
                return;
            }

            Socket i = null;

            try
            {
                i = GetServer().EndAccept(Result);

                if (ScopePriority(i) > 0)
                {
                    goto CLOSE;
                }
                else
                {
                    goto OPEN;
                }
            }
            catch
            {
                goto CLOSE;
            }

            OPEN:
            {
                GetFactory().HandleSocket(i);
                goto AQAIN;
            }

            CLOSE:
            {
                i.Close();
                i = null;
                goto AQAIN;
            }

            AQAIN:
            {
                fState = GetServer().BeginAccept(GetAsync(), fState).AsyncState;
            }
        }

        protected virtual ushort ScopePriority(Socket mSock)
        {
            ushort i = new ushort();

            try
            {
                mSock.SetIPProtectionLevel(IPProtectionLevel.EdgeRestricted);
            }
            catch
            {
                i++;
            }

            try
            {
                mSock.SetSocketOption(SocketOptionLevel.Tcp, SocketOptionName.AcceptConnection, true);
            }
            catch
            {
                i++;
            }

            if (fBlockedIps.Contains(mSock.RemoteEndPoint.ToString().Split(':')[0]))
            {
                i++;
            }

            fDossLocker.Add(mSock.RemoteEndPoint.ToString().Split(':')[0]);

            int Count = new int();

            foreach (string IP in fDossLocker)
            {
                if (IP.ToLower() == mSock.RemoteEndPoint.ToString().Split(':')[0].ToLower())
                {
                    Count++;
                }
            }

            if (Count >= 10)
            {
                if (!fBlockedIps.Contains(mSock.RemoteEndPoint.ToString().Split(':')[0]))
                {
                    fBlockedIps.Add(mSock.RemoteEndPoint.ToString().Split(':')[0]);

                    if (File.Exists("blocked_ips.txt"))
                    {
                        using (StreamWriter Writer = new StreamWriter("blocked_ips.txt"))
                        {
                            Writer.WriteLine(mSock.RemoteEndPoint.ToString().Split(':')[0]);
                            Writer.Dispose();
                        }
                    }
                }

                i++;
            }

            return i;
        }

        internal void CleanDDossers()
        {
            fDossLocker = new SecurityList<string>();
        }

        internal void Dispose()
        {
            if (GetServer() != null)
            {
                GetServer().Close();
                GetServer().Dispose();
            }

            if (fAsync != null)
            {
                fAsync = null;
            }

            if (fState != null)
            {
                fState = null;
            }

            if (GetFactory() != null)
            {
                GetFactory().Dispose();
                fFactory = null;
            }

            DuckEnvironment.WriteLine("Disposed SocketListener Succesfully!");
        }

        internal Socket GetServer()
        {
            return this.fSocket;
        }

        internal SocketFactory GetFactory()
        {
            return this.fFactory;
        }

        internal AsyncCallback GetAsync()
        {
            return this.fAsync;
        }
    }

    class SocketFactory
    {
        private SecurityDictionary<uint, SocketConnection> Sockets;

        private uint Pointer = 0;

        internal SocketFactory()
        {
            Sockets = new SecurityDictionary<uint, SocketConnection>();
        }

        private Boolean CheckConnection(Socket Sock)
        {
            if (Sock == null)
            {
                return false;
            }

            if (Sock.ReceiveBufferSize != 8192 && Sock.SendBufferSize != 8192)
            {
                return false;
            }

            if (Sock.Connected && Sock.ProtocolType == ProtocolType.Tcp && Sock.SocketType == SocketType.Stream) // Checks if could used
            {
                return true;
            }

            return true;
        }

        internal SocketConnection GetConnection(uint Id)
        {
            try { return Sockets[Id]; }
            catch { return null; }
        }

        internal void HandleSocket(Socket Sock)
        {
            if (!CheckConnection(Sock)) return;

            uint Id = Pointer++;

            Sock = new Socket(Sock.DuplicateAndClose(Process.GetCurrentProcess().Id));

            DuckEnvironment.WriteLine("Accepted Connection: " + Sock.RemoteEndPoint, ConsoleColor.DarkGreen);

            Sockets.Add(Id, new SocketConnection(Id,Sock));
            Sockets[Id].Start();
        }

        internal void DisposeSocket(uint Id)
        {
            DisposeSocket(Id, true);
        }

        internal void DisposeSocket(uint Id, bool Dispose)
        {
            if (GetConnection(Id) == null) return;

            Sockets.Remove(Id);

            SocketConnection Connection = GetConnection(Id);

            if (Connection != null && Connection.GetSocket() != null)
            {
                DuckEnvironment.WriteLine("Disposed Connection: " + Connection.GetSocket().RemoteEndPoint, ConsoleColor.DarkGreen);
            }

            if (Dispose && Connection != null)
            {
                Connection.Dispose();
            }

            Connection = null;
        }

        internal void Dispose()
        {
            if (Sockets != null)
            {
                Sockets.Keys.ToList().ForEach(DisposeSocket);
                Sockets = null;
            }
        }
    }
}
