﻿using System;

using MySql.Data.MySqlClient;

using DuckEmulator.HabboHotel.Inside.Storage.MySql.Database.Exceptions;
using DuckEmulator.HabboHotel.Inside.Storage.MySql.Database.Interfaces;

namespace DuckEmulator.HabboHotel.Inside.Storage.MySql.Database.SessionDetails
{
    internal class TransactionQueryReactor : QueryAdapter, IQueryAdapter, IRegularQueryAdapter, IDisposable
    {
        private bool finishedTransaction;
        private MySqlTransaction transaction;

        internal TransactionQueryReactor(DatabaseClient client) : base(client)
        {
            initTransaction();
        }

        #region IQueryAdapter Members

        public void Dispose()
        {
            if (!finishedTransaction)
            {
                throw new TransactionException(
                    "The transaction needs to be completed by commit() or rollback() before you can dispose this item.");
            }
            
            command.Dispose();
            client.reportDone();
        }

        public void doCommit()
        {
            try
            {
                transaction.Commit();
                finishedTransaction = true;
            }
            catch (MySqlException exception)
            {
                throw new TransactionException(exception.Message);
            }
        }

        public void doRollBack()
        {
            try
            {
                transaction.Rollback();
                finishedTransaction = true;
            }
            catch (MySqlException exception)
            {
                throw new TransactionException(exception.Message);
            }
        }

        #endregion

        internal bool getAutoCommit()
        {
            return false;
        }

        private void initTransaction()
        {
            command = client.getNewCommand();
            transaction = client.getTransaction();
            command.Transaction = transaction;
            command.Connection = transaction.Connection;
            finishedTransaction = false;
        }
    }
}