﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using DuckEmulator.Utilities;

namespace DuckEmulator.HabboHotel.Outside.Messages.Types
{
    class ServerMessage
    {
        private uint MessageId;

        internal uint Id
        {
            get
            {
                return MessageId;
            }
        }

        private List<byte> Body;

        internal string Header
        {
            get
            {
                return Encoding.ASCII.GetString(Base64Encoding.Encodeuint(MessageId, 2));
            }
        }

        internal int Length
        {
            get
            {
                return Body.Count;
            }
        }

        internal ServerMessage() { }

        internal ServerMessage(uint _MessageId)
        {
            Init(_MessageId);    
        }

        public override string ToString()
        {
            return Header + Encoding.ASCII.GetString(Body.ToArray());
        }

        internal string ToBodyString()
        {
            return Encoding.ASCII.GetString(Body.ToArray());
        }

        internal void Clear()
        {
            Body = new List<byte>();
        }

        internal void Init(uint _MessageId)
        {
            MessageId = _MessageId;
            Clear();
        }

        internal void AppendByte(byte b)
        {
            Body.Add(b);
        }

        internal void AppendBytes(byte[] Data)
        {
            if (Data == null || Data.Length == 0)
            {
                return;
            }

            Body.AddRange(Data);
        }

        internal void AppendString(string s, Encoding encoding)
        {
            if (s == null || s.Length == 0)
            {
                return;
            }

            AppendBytes(encoding.GetBytes(s));
        }

        internal void AppendString(string s)
        {
            AppendString(s, Encoding.ASCII);
        }

        internal void AppendStringWithBreak(string s)
        {
            AppendStringWithBreak(s, 2);
        }

        internal void AppendStringWithBreak(string s, byte BreakChar)
        {
            AppendString(s);
            AppendByte(BreakChar);
        }

        internal void AppendInt32(Int32 i)
        {
            AppendBytes(WireEncoding.EncodeInt32((ushort)i));
        }

        internal void AppendRawInt32(Int32 i)
        {
            AppendString(i.ToString(), Encoding.ASCII);
        }

        internal void AppendUInt(uint i)
        {
            ushort _i = (ushort)i;
            AppendInt32(_i);
        }

        public void AppendRawUInt(uint i)
        {
            ushort _i = (ushort)i;
            AppendRawInt32(_i);
        }

        internal void AppendBoolean(Boolean Bool)
        {
            if (Bool)
            {
                Body.Add(WireEncoding.POSITIVE);
                return;
            }

            Body.Add(WireEncoding.NEGATIVE);
        }

        internal byte[] GetBytes()
        {
            byte[] Data = new byte[Length + 3];
            byte[] Header = Base64Encoding.Encodeuint(MessageId, 2);

            Data[0] = Header[0];
            Data[1] = Header[1];

            for (int i = 0; i < Length; i++)
            {
                Data[i + 2] = Body[i];
            }

            Data[Data.Length - 1] = 1;

            return Data;
        }
    }
}
